//+------------------------------------------------------------------+
//|                                        Custom Moving Average.mq4 |
//|                      Copyright  2004, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2004, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"

#property indicator_separate_window
#property indicator_minimum -10
#property indicator_maximum 110
#property indicator_level1 0
#property indicator_level2 100
#property indicator_buffers 3
#property indicator_color1 Red
#property indicator_color2 DarkBlue
#property indicator_style2 STYLE_DASH
#property indicator_color3 Black
#property indicator_style3 STYLE_DOT
//---- indicator parameters
extern int StochCount=7;
extern int Stoch_D=12;
extern int Stoch_K=8;
extern int Stoch_S=5;
extern double Koef=1.2;
extern bool ShowAverage = true;
extern bool ShowStoch = true;
//---- indicator buffers
double Buf[], Average[], Stoch[];
double D[], K[], S[];

int draw_begin1=0;
int draw_begin2=0;

//+------------------------------------------------------------------+
void CloseGroupingOfShots(double Massiv[], double& res1, double& res2)
{
	res1 = 0; res2 = 0;
	int i = 0;
	double j = 0;
	double Min = 0, Max = 0, Sred = 0;
	double tmp;
	
	j = ArraySize(Massiv);
	if (j == 0)
		return(0);
	for (i = 0; i < j; i++)
	{
		tmp = Massiv[i];
		Sred += tmp;
		if (i == 0)
		{
			Min = tmp;
			Max = tmp;
		} else {
			Min = MathMin(Min, tmp);
			Max = MathMax(Max, tmp);
		}
	}
	Sred /= j;
	if ( (Sred == 0) || (Max == 0) )
		return(0);
	
	res1 = 100 - (Max - Min);
	res2 = Sred;
}
//+------------------------------------------------------------------+
void GetResultForInstrument(string Instrument, int timeframe, datetime ontime, double& var1, double& var2, double& var3)
{
	int shift = iBarShift(Instrument, timeframe, ontime, false);
	double Value[];
	ArrayResize(Value, StochCount);
	for (int i = 0; i < StochCount; i++)
	 {
		Value[i] = iStochastic(Instrument, timeframe, D[i], K[i], S[i], 3, 1, 0, shift);
		if (i == (StochCount-1))
			var3 = Value[i];
	 }
	double res1 = 0;
	double res2 = 0;
	CloseGroupingOfShots(Value, res1, res2);
	var1 = res1;
	var2 = res2;
}
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   IndicatorDigits(1);
   string short_name=" Stochastic X8";
   IndicatorShortName(short_name+"*"+DoubleToStr(Koef,1)+" ("+Stoch_D+", "+Stoch_K+", "+Stoch_S+")");
   
   IndicatorBuffers(3);
   SetIndexBuffer(0, Buf);
   SetIndexLabel(0, "Stoch ("+Ds_0(D[0])+", "+Ds_0(K[0])+", "+Ds_0(S[0])+")\n");
   SetIndexBuffer(1, Average);
   SetIndexLabel(1, "Average");
   SetIndexBuffer(2, Stoch);
   SetIndexLabel(2, "Stoch");

	ArrayResize(D, StochCount);
	ArrayResize(K, StochCount);
	ArrayResize(S, StochCount);
	for (int i = 0; i < StochCount; i++)
	 {
	 	if (i == 0)
	 	 {
	 	 	D[0] = Stoch_D;
	 	 	K[0] = Stoch_K;
	 	 	S[0] = Stoch_S;
	 	 } else {
	 	 	D[i] = D[i-1] * Koef;
	 	 	K[i] = K[i-1] * Koef;
	 	 	S[i] = S[i-1] * Koef;
	 	 }
	 }
   
	//---- initialization done
   draw_begin1 = Stoch_K + Stoch_S;
   draw_begin2 = draw_begin1 + Stoch_D;
   
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
  {
   int counted_bars = IndicatorCounted();
   if (Bars <= draw_begin2)
   	return(0);
   int i1 = Bars - Stoch_K;
   if (counted_bars > Stoch_K) i1 = Bars - counted_bars - 1;
   int i2 = Bars - draw_begin1;
   if (counted_bars > draw_begin1) i2 = Bars - counted_bars - 1;
   int iend = MathMax(i1, i2);

	double var1, var2, var3;
   for(int i = 0; i < iend; i++)
   {
	 	GetResultForInstrument(Symbol(), 0, Time[i], var1, var2, var3);
	 	Buf[i] = var1;
	 	if (ShowAverage)
	 		Average[i] = var2;
	 	if (ShowStoch)
	 		Stoch[i] = var3;
    }
   return(0);
  }
//+------------------------------------------------------------------+
string Ds_0(double DOUBLE) {return(DoubleToStr(DOUBLE,0));}
//+------------------------------------------------------------------+

